#!/usr/bin/python
# html output lib

import string
import options as Options
from consts import *
from __main__ import process_output

__all__ = [
  'loaded',
  'link',
  'write_content',
  'write_sh_link',
  'write_index',
  'write_root',
  'start_text',
  'end_text',
  'start_code',
  'end_code'
]


def loaded():
  return True


def html_header(title):
  return '<?xml version="1.0" encoding="utf-8" ?>\n<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"\n"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd" >\n<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">\n<head><title>%s</title></head>\n<body><h1><a name="top">%s</a></h1><hr />\n' % (title, title)


def start_text():
  if Options.Debug: print('  start_text <p>')
  return '\n<p>'


def end_text():
  if Options.Debug: print('  end_text </p>')
  return '</p>\n'


def start_code():
  if Options.Debug: print('  start_code <pre>')
  return '\n<pre>'


def end_code():
  if Options.Debug: print('  end_code </pre>')
  return '</pre>\n'


def link(word, target):
  return '<a href="#%s">%s</a>' % (target, word)


def end_page():
  return '</body></html>\n'


def process_text(text):
  a = ''

  text = text.strip('/* ')
  if len(text) >= 1 and text[0] == '+':
    a = '</p><p>'
    text = text[1:]

  text = text.lstrip(' ')
  if len(text) == 0: return ''

  text = text.replace('&', '&amp;')
  text = text.replace('<', '&lt;')
  text = text.replace('>', '&gt;')
  text = text.replace('"', '&quot;')
  text = text.replace("'", '&apos;')

  text = text.replace('{fcode}', '<span style="font-family:monospace">')
  text = text.replace('{f}', '</span>')

  return a + text + ' '


def process_code(text):

  text = text.replace('/*', '')
  text = text.replace('*/', '')
#  text = text.lstrip('* ')

  text = text.replace('&', '&amp;')
  text = text.replace('<', '&lt;')
  text = text.replace('>', '&gt;')

  return text + '\n'


def blank_line(n):
  return '<br />\n'


def write_sh_link(page, tag):
  return


def write_content(out, page, data, lib, index_num):

  out.write('<hr width="50%" align="left" />')
  out.write('<h3 align="left"><a name="%s">%s</a></h3>\n' % (page, page))
  out.write(data + '\n')
  out.write('<p><a href="#top">back to top</a></p>\n')

  if Options.Debug: print('page: %s' % page)
  return


def tabulate_links(keys, items, cols, out, local):
  i = 0

  out.write('<table border="0" cellpadding="4" cellspacing="0">\n')

  for k in keys:

    try:
      tag = items[k]
    except KeyError:
      continue

    if Options.Debug: print('%d %s => %s' % (i, k, tag))

    if i % cols == 0:
      if i > 0: out.write('</tr>')
      out.write('<tr>\n')

    if local:
      out.write('<td><a href="#%s">%s</a></td>\n' % (tag, k))
    else:
      out.write('<td><a href="%s">%s</a></td>\n' % (tag, k))

    i += 1
    # if i % cols == 0: out.write('</tr>')

  while i % cols != 0:
    out.write('<td>&nbsp;</td>')
    i += 1

  out.write('</tr></table>\n')

  return


def sub_index(out, indexes, data, type):

  if not Options.FullIndex:
    keys = list(data.keys())			# list of unique tags
  else:
    keys = list(indexes[type].keys())		# list of all page names

  keys.sort()
  tabulate_links(keys, indexes[type], Options.IndexCols[type], out, True)
  return


def write_index(indexes, data, lib):

  out = open('%s.%s/html' % (Options.OutputDir, lib), 'w')
  out.write(html_header('%s.h' % lib))

  # write index text
  if '__index' in list(data.keys()):
    out.write('%s\n<hr />\n' % process_output('', data['__index'], indexes))

  if len(indexes[TYPE_TYPEDEF]) > 0:
    out.write('<h2>Types in this header file</h2>\n')
    sub_index(out, indexes, data, TYPE_TYPEDEF);

  if len(indexes[TYPE_FN]) > 0:
    out.write('<h2>Functions in this header file</h2>\n')
    sub_index(out, indexes, data, TYPE_FN);

  if len(indexes[TYPE_DEF]) > 0:
    out.write('<h2>Macros in this header file</h2>\n')
    sub_index(out, indexes, data, TYPE_DEF);

  if len(indexes[TYPE_PRAGMA]) > 0:
    out.write('<h2>Pragmas in this header file</h2>\n')
    sub_index(out, indexes, data, TYPE_PRAGMA);

  if len(indexes[TYPE_NOTE]) > 0:
    out.write('<h2>Notes in this header file</h2>\n')
    sub_index(out, indexes, data, TYPE_NOTE);

  return out


# generate root (index of indexes) page
def write_root(libs):

  inf = open(Options.RootPage, 'rb')
  #out = open('%s.index/html' % Options.OutputDir, 'wb')
  out = open('%s.index/html' % Options.OutputDir, 'w')

  title = inf.readline()
  inf.close()
  out.write(html_header('Version %s' % title))
  out.write('<h2>Header files in this manual</h2>')

  items = {}
  for lib in libs:
    if lib != '':
      items['%s.h' % lib] = '%s.html' % lib

  if Options.Debug: print(items)

  sortedlist = list(items.keys())
  sortedlist.sort()

  tabulate_links(sortedlist, items, 4, out, False)

  out.write(end_page())

  out.close()
  out = None

  return
