#!/usr/bin/python
# stronghelp output lib

import string
import options as Options
from consts import *
from __main__ import process_output

__all__ = [
  'loaded',
  'link',
  'write_content',
  'write_sh_link',
  'write_index',
  'write_root',
  'start_text',
  'end_text',
  'start_code',
  'end_code'
]


def loaded():
  return True


def start_text():
  return '#wrap on\n'


def end_text():
  return '\n#wrap off\n\n'


def start_code():
  return '{fcode}'


def end_code():
  return '{f}\n'


def link(word, target):
  return '<%s>' % word


def end_page():
  return ''


def outputpath(file):

  c = file[0:1].lower()

  if c in 'abcdefghijklmnopqrstuvwxyz_':
    return '%s.[%s].%s' % (Options.OutputDir, c, file)

  return '%s.%s' % (Options.OutputDir, file)


def write_content(out, page, data, lib, index_num):
  if page == None or page == '':
    print('write_content: warning: NULL page in lib %s\n' % lib)
    return

 # if index_num == TYPE_NOTE: mode = 'wb'
 # else: mode = 'w'
  mode = 'w'

  if Options.Debug: print('write content page: %s' % page)
  if Options.Debug: print('write content lib: %s' % lib)
  out = open(outputpath(page), mode)
  out.write('%s\n#Parent %s/h\n#wrap off\n' % (page, lib))
  out.write(data.rstrip())
  out.close()

#  if index_num == TYPE_NOTE: system('settype %s data' % outputpath(page))

  if Options.Debug: print('page: %s' % page)
  return


def write_sh_link(page, tag):
  if page == None or page == '':
    print('write_link: warning: NULL page (tag %s)\n' % tag)
    return

  if Options.Debug: print('link: %s -> %s' % (page, tag))
  open("%s>%s" % (outputpath(page), tag), 'w').close
  return


def process_text(text):
  a = ''

  text = text.strip('/*\t ')
  if len(text) >= 1 and text[0] == '+':
    a = '\n\n'
    text = text[1:]

  text = text.lstrip(' ')
  if len(text) == 0: return ''

  text = text.replace(' >', ' \>')
  text = text.replace('< ', '\< ')
  text = text.replace('#', '\#')
#  text = text.replace('{', '\{')

  return a + text + ' '


def process_code(text):

  text = text.replace('/*', '')
  text = text.replace('*/', '')
#  text = text.lstrip('* ')

  text = text.replace('>', '\>')
  text = text.replace('<', '\<')
  text = text.replace('#', '\#')
  text = text.replace('{', '\{')
  return text + '\n'


def blank_line(n):

  txt = ''
  while n > 0:
    txt += '\n'
    n -= 1

  return txt


def tabulate_links(keys, items, cols, out):
  i = 0

  for k in keys:

    try:
      tag = items[k]
    except KeyError:
      continue

    if Options.Debug: print('%d %s => %s' % (i, k, tag))

    out.write('<%s=>%s>\t' % (k, tag))
    i += 1
    if i % cols == 0: out.write('\n')

  if i % cols != 0: out.write('\n')
  return


def sub_index(out, indexes, data, type):

  if not Options.FullIndex:
    keys = list(data.keys())			# list of unique tags
  else:
    keys = list(indexes[type].keys())		# list of all page names

  keys.sort()
  tabulate_links(keys, indexes[type], Options.IndexCols[type], out)
  return


# indexes and data are dict objects
def write_index(indexes, data, lib):

  # Write index for this file
  out = open(outputpath('%s/h' % lib), 'w')
  out.write('%s.h\n' % lib)

  # write index text
  if '__index' in list(data.keys()):
    out.write('%s\n' % process_output('', data['__index'], indexes))

  out.write('#wrap off')

  if len(indexes[TYPE_TYPEDEF]) > 0:
    out.write('\nTypes in this header file:\n#Tab\n')
    sub_index(out, indexes, data, TYPE_TYPEDEF)

  if len(indexes[TYPE_FN]) > 0:
    out.write('\nFunctions in this header file:\n#Tab\n')
    sub_index(out, indexes, data, TYPE_FN)

  if len(indexes[TYPE_DEF]) > 0:
    out.write('\nMacros defined in this header file:\n#Tab\n')
    sub_index(out, indexes, data, TYPE_DEF)

  if len(indexes[TYPE_PRAGMA]) > 0:
    out.write('\nPragmas defined in this header file:\n#Tab\n')
    sub_index(out, indexes, data, TYPE_PRAGMA)

  if len(indexes[TYPE_NOTE]) > 0:
    out.write('\nNotes defined in this header file:\n#Tab\n')
    sub_index(out, indexes, data, TYPE_NOTE)

  out.close()

  return None


# generate root page
def write_root(libs):

  items = {}
  for lib in libs:
    if lib != '':
      items['%s.h' % lib] = '%s/h' % lib

  if Options.Debug: print(items)

  sortedlist = list(items.keys())
  sortedlist.sort()

  with open('%s.!Root' % Options.OutputDir, mode='w') as out:
    with open(Options.RootPage, mode='r') as inf:
      for line in inf:
        if '%index' in line:
          tabulate_links(sortedlist, items, 4, out)
        else:
          out.write(line)

    out.write(end_page())

  return
